%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%% forecasting error of alternative methods %%%%%%
% y is log(RV)
% delta is sampling interval
% para is the parameter
% T is rolling window size
% k is forecasting horizon
% i records the positions of observed values across the entire interval

function [err] = fore_err(y,delta,para,T,k,i)

H=para(1);

%% Infinite-past forecasting formulae
% right Riemann sum
seq=1:1:T-1;
[coef] = w1(T*delta,seq'*delta,k*delta,H);
coef=[coef;coef(end)];
weight1=coef./sum(coef);
% left Riemann sum
seq=0:1:T-1;
[coef] =  w1(T*delta,seq'*delta,k*delta,H);
weight2=coef./sum(coef);
% trapezoidal sum
seq=0:1:T-1;
[coef] =  w1(T*delta,seq'*delta,k*delta,H);
coef=[.5*coef(1,1);coef(2:end-1);coef(end);0.5*coef(end)];
weight3=coef./sum(coef);
% mid sum
seq=0.5:1:T-.5;
[coef] =  w1(T*delta,seq'*delta,k*delta,H);
weight4=coef./sum(coef);
% Gatheral
seq=0:1:T-1;
[coef] = w1(T*delta,seq'*delta,k*delta,H);
weight_1=coef./sum(coef);

%% Finite-past forecasting formulae
% right Riemann sum
seq=1:1:T-1;
[coef] = w2(T*delta,seq'*delta,k*delta,H);
coef=[coef;coef(end)];
weight5=coef./sum(coef);
% left Riemann sum
seq=1:1:T-1;
[coef] = w2(T*delta,seq'*delta,k*delta,H);
coef=[coef(1);coef];
weight6=coef./sum(coef);
% trapezoidal sum
seq=1:1:T-1;
[coef] = w2(T*delta,seq'*delta,k*delta,H);
coef=[.5*coef(1); coef(1);coef(2:end-1);coef(end);0.5*coef(end)];
weight7=coef./sum(coef);
% mid sum
seq=0.5:1:T-.5;
[coef] = w2(T*delta,seq'*delta,k*delta,H);
weight8=coef./sum(coef);
% % similar to the one by Gatheral
seq=1:1:T-1;
[coef] = w2(T*delta,seq'*delta,k*delta,H);
coef=[coef(1);coef];
weight_2=coef./sum(coef);

%% Optimal forecast
[covMatrix] = Covd_fbmd(T+k+i-1,H,delta);
covM = covMatrix(1:T+k,1:T+k);
Sigma=covM(1+k:T+k,1+k:T+k);
gamT=covM(1+k:T+k,1);
weight9=inv(Sigma)*(gamT);

%% the first element in y is y_0,
err(1,1)=sum(y(i-1+2:i-1+T+1).*weight1)-y(i-1+T+1+k);
err(2,1)=sum(y(i-1+1:i-1+T).*weight2)-y(i-1+T+1+k);
err(3,1)=sum(y(i-1+1:i-1+T+1).*weight3)-y(i-1+T+1+k);
err(4,1)=sum(y(i-1+2:i-1+T+1).*weight4)-y(i-1+T+1+k);
err(5,1)=sum(y(i-1+2:i-1+T+1).*weight_1)-y(i-1+T+1+k);

err(6,1)=sum(y(i-1+2:i-1+T+1).*weight5)-y(i-1+T+1+k);
err(7,1)=sum(y(i-1+1:i-1+T).*weight6)-y(i-1+T+1+k);
err(8,1)=sum(y(i-1+1:i-1+T+1).*weight7)-y(i-1+T+1+k);
err(9,1)=sum(y(i-1+2:i-1+T+1).*weight8)-y(i-1+T+1+k);
err(10,1)=sum(y(i-1+2:i-1+T+1).*weight_2)-y(i-1+T+1+k);

err(11,1)=sum(flip(y(i-1+2:i-1+T+1)).*weight9)-y(i-1+T+1+k);

end