%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%% Portfolio allocation %%%%%%
% RV is the RV
% RVf is the rolling window forecasted RV (fBm or HAR)
% rm is daily market return over the entire sample period
% rf is the daily risk free rate
% riskAversion parameter
% Bandi and Russell (2006): The annual risk-free return rf is set to 0.06 (converted to daily value by dividing
% by 365)

function [utility, weight, rp]=...
    PortfolioAllocation(RV,RVf,rm,rf,riskAver,flag_tc,flag_shortsale,flag_leverage,h)

n=length(RV);
rmm=mean(rm);
delta=mean(rm.^2)/mean(RV);  % adjusting factor for overnight returns
weight=(rmm-rf)./(riskAver*delta*RVf);


if flag_shortsale==0
    weight=weight.*(weight>0);
end

if flag_shortsale==1
     ind=(weight<-.5);
    weight(ind)=-.5;
end

if flag_leverage==0
    ind=(weight>1);
    weight(ind)=1;
end

if flag_leverage==1
    ind=(weight>1.5);
    weight(ind)=1.5;
end

rp=weight.*rm+(ones(n,1)-weight)*rf;

if flag_tc==1
    rho=0.0015;
    dw=diff(weight);
    dw=[weight(1);dw];
    rp_adj=rp-rho*(1+rp).*abs(dw);
    rp=rp_adj;
end

utility=mean(rp)-riskAver/2*(std(rp)).^2;
utility=utility/h*252;
end

