%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%% parameters of fBm %%%%%%
% x is log(RV)
% delta is sampling interval

function H =fbm_est(x,delta)

x2=x(3:end)-x(1:end-2);
x22=x2(3:end)-x2(1:end-2);
x1=x(2:end)-x(1:end-1);
x11=x1(2:end)-x1(1:end-1);
Hhat=log2(sum(x22.^2)/sum(x11.^2))/2;
sigma2=(sum(x11.^2)/size(x,1)/(4-2^(2*Hhat))/delta^(2*Hhat));
%  obtain the time-domain mle
options=optimset('fminsearch');
% define objective function
fun=@(p) likelihood_mle_f(x,delta,p);
% find parameter with a good initial value
H = fminsearch(fun,[.2;1]); 

end

function L=likelihood_mle_f(x,delta,p)
%% this is objective function of time-domain likelihood
H=p(1);
sigma2=p(2);

if H>0
n=size(x,1);

% compute the covariance matrix
[covMatrix] = Covd_fbmd(n,H,delta);

covMatrix=covMatrix*sigma2;

% compute the eigenvalues of the covariance matrix
lam=eig(covMatrix);

% compute the likelihood
L=1/2/n*sum(log(lam))+1/2/n*x'*inv(covMatrix)*x;
else
    
L=Inf;
end

end