%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%% weights in alternative forecasting formulae %%%%%%
% T is rolling window size
% k is forecasting horizon
% H is Hurst parameter

function w=weigh(T,k,H)
%% Infinite-past forecasting formulae
% right Riemann sum
seq=1:1:T-1;
[coef] = w1(T,seq',k,H);
coef=[coef;coef(end)];
weight1=coef./sum(coef);
% left Riemann sum
seq=0:1:T-1;
[coef] = w1(T,seq',k,H);
weight2=coef./sum(coef);
% trapezoidal sum
seq=0:1:T-1;
[coef] = w1(T,seq',k,H);
coef=[.5*coef(1,1);coef(2:end-1);coef(end);0.5*coef(end)];
weight3=coef./sum(coef);
% mid sum
seq=0.5:1:T-.5;
[coef] = w1(T,seq',k,H);
weight4=coef./sum(coef);
% Gatheral
seq=0:1:T-1;
[coef] = w1(T,seq',k,H);
weight_1=coef./sum(coef);

%% Finite-past forecasting formulae
% right Riemann sum
seq=1:1:T-1;
[coef] = w2(T,seq',k,H);
coef=[coef;coef(end)];
weight5=coef./sum(coef);
% left Riemann sum
seq=1:1:T-1;
[coef] = w2(T,seq',k,H);
coef=[coef(1);coef];
weight6=coef./sum(coef);
% trapezoidal sum
seq=1:1:T-1;
[coef] = w2(T,seq',k,H);
coef=[.5*coef(1); coef(1);coef(2:end-1);coef(end);0.5*coef(end)];
weight7=coef./sum(coef);
% mid sum
seq=0.5:1:T-.5;
[coef] = w2(T,seq',k,H); 
weight8=coef./sum(coef);
% % similar to the one by Gatheral
seq=1:1:T-1;
[coef] = w2(T,seq',k,H);
coef=[coef(1);coef];
weight_2=coef./sum(coef);

%% Optimal forecast
[covMatrix] = Covd_fbm(T+k,H);
Sigma=covMatrix(1+k:T+k,1+k:T+k);
gamT=covMatrix(1+k:T+k,1);
weight9=inv(Sigma)*(gamT);

w{1}=weight1;
w{2}=weight2;
w{3}=weight3;
w{4}=weight4;
w{5}=weight_1;
w{6}=weight5;
w{7}=weight6;
w{8}=weight7;
w{9}=weight8;
w{10}=weight_2;
w{11}=weight9;

end